@@ -28,7 +28,7 @@
 #include <sofia-sip/sresolv.h>
 
 #ifdef HAVE_LIBIPHB
-#include <libiphb.h>
+#include <iphbd/libiphb.h>
 #endif
 
 struct _TpsipConnectionPrivate
@@ -69,6 +69,7 @@ struct _TpsipConnectionPrivate
   gboolean loose_routing;
   gboolean discover_binding;
   gboolean immutable_streams;
+  gboolean ignore_tls_errors;
 
   gboolean keepalive_interval_specified;
 
