@@ -49,6 +49,7 @@
 #include "sip-connection-sofia.h"
 
 #include <sofia-sip/msg_header.h>
+#include <sofia-sip/tport_tag.h>
 
 #define DEBUG_FLAG TPSIP_DEBUG_CONNECTION
 #include "debug.h"
@@ -102,6 +103,7 @@ enum
   PROP_LOCAL_PORT,         /**< Local port for SIP (normally not needed, chosen by stack) */
   PROP_EXTRA_AUTH_USER,	   /**< User name to use for extra authentication challenges */
   PROP_EXTRA_AUTH_PASSWORD,/**< Password to use for extra authentication challenges */
+  PROP_IGNORE_TLS_ERRORS,  /**< If true, TLS errors will be ignored */
   PROP_SOFIA_ROOT,         /**< Event root pointer from the Sofia-SIP stack */
   LAST_PROPERTY
 };
@@ -303,6 +305,9 @@ tpsip_connection_set_property (GObject      *object,
     priv->extra_auth_password =  g_value_dup_string (value);
     break;
   }
+  case PROP_IGNORE_TLS_ERRORS:
+    priv->ignore_tls_errors = g_value_get_boolean (value);
+    break;
   case PROP_SOFIA_ROOT: {
     priv->sofia_root = g_value_get_pointer (value);
     break;
@@ -392,6 +397,9 @@ tpsip_connection_get_property (GObject      *object,
     g_value_set_uint (value, priv->local_port);
     break;
   }
+  case PROP_IGNORE_TLS_ERRORS:
+    g_value_set_boolean (value, priv->ignore_tls_errors);
+    break;
   case PROP_SOFIA_ROOT: {
     g_value_set_pointer (value, priv->sofia_root);
     break;
@@ -523,7 +531,7 @@ tpsip_connection_class_init (TpsipConnectionClass *klass)
       "Interval between keepalive probes in seconds "
       "(0 = disabled, unset = use a default interval)",
       0, G_MAXUINT32, 0,
-      G_PARAM_CONSTRUCT | G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
+      G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
   INST_PROP(PROP_KEEPALIVE_INTERVAL);
 
   param_spec = g_param_spec_boolean ("discover-binding",
@@ -584,6 +592,12 @@ tpsip_connection_class_init (TpsipConnectionClass *klass)
       G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
   INST_PROP(PROP_EXTRA_AUTH_PASSWORD);
 
+  param_spec = g_param_spec_boolean ("ignore-tls-errors", "Ignore TLS errors",
+      "If true, the TLS verification errors will be ignored",
+      FALSE,
+      G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
+  INST_PROP(PROP_IGNORE_TLS_ERRORS);
+
 #undef INST_PROP
 
   tp_dbus_properties_mixin_class_init (object_class,
@@ -929,6 +943,8 @@ tpsip_connection_start_connecting (TpBaseConnection *base,
       NUTAG_AUTOANSWER(0),
       NUTAG_APPL_METHOD("MESSAGE"),
       SIPTAG_ALLOW_STR("INVITE, ACK, BYE, CANCEL, OPTIONS, PRACK, MESSAGE, UPDATE"),
+      TAG_IF(!priv->ignore_tls_errors,
+             TPTAG_TLS_VERIFY_POLICY(TPTLS_VERIFY_ALL)),
       TAG_NULL());
   if (priv->sofia_nua == NULL)
     {
@@ -943,10 +959,10 @@ tpsip_connection_start_connecting (TpBaseConnection *base,
   tpsip_conn_update_nua_keepalive_interval (self);
   tpsip_conn_update_nua_contact_features (self);
 
-  if (priv->stun_host != NULL)
-    tpsip_conn_resolv_stun_server (self, priv->stun_host);
-  else if (priv->discover_stun)
+  if (priv->discover_stun)
     tpsip_conn_discover_stun_server (self);
+  else if (priv->stun_host != NULL)
+    tpsip_conn_resolv_stun_server (self, priv->stun_host);
 
   DEBUG("initialized a Sofia-SIP NUA at address %p", priv->sofia_nua);
 
