@@ -1,7 +1,7 @@
 /*
  * sip-text-channel.c - Source for TpsipTextChannel
  * Copyright (C) 2005-2008 Collabora Ltd.
- * Copyright (C) 2005-2008 Nokia Corporation
+ * Copyright (C) 2005-2009 Nokia Corporation
  *   @author Martti Mela <first.surname@nokia.com>
  *   @author Mikhail Zabaluev <mikhail.zabaluev@nokia.com>
  *
@@ -166,7 +166,7 @@ tpsip_text_channel_constructed (GObject *obj)
   TpsipTextChannelPrivate *priv;
   TpBaseConnection *base_conn;
   TpHandleRepoIface *contact_handles;
-  DBusGConnection *bus;
+  TpDBusDaemon *bus;
   GObjectClass *parent_object_class =
       G_OBJECT_CLASS (tpsip_text_channel_parent_class);
 
@@ -190,8 +190,8 @@ tpsip_text_channel_constructed (GObject *obj)
                     G_CALLBACK (tpsip_text_channel_nua_r_message_cb),
                     NULL);
 
-  bus = tp_get_bus();
-  dbus_g_connection_register_g_object(bus, priv->object_path, obj);
+  bus = tp_base_connection_get_dbus_daemon (base_conn);
+  tp_dbus_daemon_register_object (bus, priv->object_path, obj);
 }
 
 
@@ -831,7 +831,7 @@ tpsip_text_channel_send(TpSvcChannelTypeText *iface,
       GError invalid =  { TP_ERRORS, TP_ERROR_NOT_AVAILABLE,
           "Invalid recipient" };
 
-      g_warning ("invalid recipient handle %d", priv->handle);
+      WARNING ("invalid recipient handle %d", priv->handle);
       dbus_g_method_return_error (context, &invalid);
       return;
     }
@@ -888,7 +888,7 @@ tpsip_text_channel_nua_r_message_cb (TpsipTextChannel *self,
   /* Shouldn't happen... */
   if (node == NULL)
     {
-      g_warning ("message pending sent acknowledgement not found");
+      WARNING ("message pending sent acknowledgement not found");
       return FALSE;
     }
 
